/*
This file is part of myFitter, a C++ library for maximum likelihood fits.
Copyright (C) 2012, 2013  Martin Wiebusch

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef MYFITTER_UBLASUTILS_DETERMINANT_HPP
#define MYFITTER_UBLASUTILS_DETERMINANT_HPP

#include <boost/numeric/ublas/vector.hpp>
#include <boost/numeric/ublas/vector_proxy.hpp>
#include <boost/numeric/ublas/matrix.hpp>
#include <boost/numeric/ublas/triangular.hpp>
#include <boost/numeric/ublas/lu.hpp>
#include <boost/numeric/ublas/io.hpp>

namespace myfitter { namespace ublasutils {

template<class T>
T det(const boost::numeric::ublas::matrix<T>& m) {
    using boost::numeric::ublas::matrix;
    using boost::numeric::ublas::permutation_matrix;
    using boost::numeric::ublas::lu_substitute;
    
    BOOST_ASSERT(m.size1() == m.size2());
    
          // create a working copy of the input
    matrix<T> mLu(m);
    permutation_matrix<typename matrix<T>::size_type> pivots(m.size1());
    
    lu_factorize(mLu, pivots);
    
    T det = 1.0;
    
    for (typename matrix<T>::size_type i=0; i < pivots.size(); ++i) {
        if (pivots(i) != i)
            det *= -1.0;
        det *= mLu(i,i);
    }
    return det;
}

}}


#endif
