/*
This file is part of myFitter, a C++ library for maximum likelihood fits.
Copyright (C) 2012, 2013  Martin Wiebusch

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef MYFITTER_UBLASUTILS_MATRIX_VECTOR_PRINT_HPP
#define MYFITTER_UBLASUTILS_MATRIX_VECTOR_PRINT_HPP

#include <iostream>
#include <iomanip>

#include <boost/numeric/ublas/matrix_expression.hpp>
#include <boost/numeric/ublas/vector_expression.hpp>

namespace myfitter { namespace ublasutils { 

    template<class T>
    void print_complex_matrix(std::ostream& os, 
                              const boost::numeric::ublas::matrix_expression<T>& A,
                              std::streamsize width, std::streamsize prec)
    {
        typedef typename T::size_type size_type;

        std::ios::fmtflags oldflags = os.flags();

        os.setf(std::ios::fixed, std::ios::floatfield);

        for(size_type i = 0; i < A().size1(); i++) {
            for(size_type j = 0; j < A().size2(); j++)
                os << '('
                   << std::setw(width) << std::fixed << std::setprecision(prec)
                   << std::real(A()(i,j))
                   << ','
                   << std::setw(width) << std::fixed << std::setprecision(prec)
                   << std::imag(A()(i,j)) << ") ";
     
            os << std::endl;
        }

        os.flags(oldflags);
    }

    template<class T>
    void print_real_matrix(std::ostream& os, 
                           const boost::numeric::ublas::matrix_expression<T>& A,
                           std::streamsize width, std::streamsize prec)
    {
        typedef typename T::size_type size_type;

        std::ios::fmtflags oldflags = os.flags();

        os.setf(std::ios::fixed, std::ios::floatfield);

        for(size_type i = 0; i < A().size1(); i++) {
            for(size_type j = 0; j < A().size2(); j++)
                os << std::setw(width) << std::fixed << std::setprecision(prec)
                   << A()(i,j) << " ";
     
            os << std::endl;
        }

        os.flags(oldflags);
    }


    template<class T>
    void print_complex_vector(std::ostream& os, 
                              const boost::numeric::ublas::vector_expression<T>& v,
                              std::streamsize width, std::streamsize prec)
    {
        typedef typename T::size_type size_type;

        std::ios::fmtflags oldflags = os.flags();
        os.setf(std::ios::fixed, std::ios::floatfield);

        for(size_type i = 0; i < v().size(); i++)
            os << '('
               << std::setw(width) << std::fixed << std::setprecision(prec)
               << std::real(v()(i))
               << ','
               << std::setw(width) << std::fixed << std::setprecision(prec)
               << std::imag(v()(i)) << ") ";

        os << std::endl;

        os.flags(oldflags);
    }

    template<class T>
    void print_real_vector(std::ostream& os, 
                           const boost::numeric::ublas::vector_expression<T>& v,
                           std::streamsize width, std::streamsize prec)
    {
        typedef typename T::size_type size_type;

        std::ios::fmtflags oldflags = os.flags();
        os.setf(std::ios::fixed, std::ios::floatfield);

        for(size_type i = 0; i < v().size(); i++)
            os << std::setw(width) << std::fixed << std::setprecision(prec)
               << v()(i)
               << " ";

        os << std::endl;

        os.flags(oldflags);
    }

}}

#endif
